@@ -89,11 +89,11 @@ struct obc_session {
 	uint8_t channel;
 	struct obc_transport *transport;
 	struct obc_driver *driver;
-	gchar *path;		/* Session path */
+	char *path;		/* Session path */
 	DBusConnection *conn;
 	GObex *obex;
 	struct pending_request *p;
-	gchar *owner;		/* Session owner */
+	char *owner;		/* Session owner */
 	guint watch;
 	GQueue *queue;
 	guint queue_complete_id;
@@ -934,7 +934,7 @@ guint obc_session_setpath(struct obc_session *session, const char *path,
 	p = pending_request_new(session, NULL, setpath_complete, data);
 
 	/* Relative path */
-	if (path[0] != '/') {
+	if (path[0] != '/' && path[0] != 0) {
 		first = data->remaining[data->index];
 		data->index++;
 	}
