@@ -36,7 +36,6 @@
 #include "log.h"
 #include "dbus-common.h"
 #include "adapter.h"
-#include "manager.h"
 #include "device.h"
 #include "eir.h"
 #include "agent.h"
@@ -108,7 +107,7 @@ static void register_agent(void)
 {
 	DBusMessage *message;
 	DBusPendingCall *call;
-	const gchar *path = AGENT_PATH;
+	const char *path = AGENT_PATH;
 
 	message = dbus_message_new_method_call(NEARD_NAME, NEARD_PATH,
 			NEARD_MANAGER_INTERFACE, "RegisterHandoverAgent");
@@ -122,18 +121,21 @@ static void register_agent(void)
 
 	if (!dbus_connection_send_with_reply(btd_get_dbus_connection(),
 							message, &call, -1)) {
+		dbus_message_unref(message);
 		error("D-Bus send failed");
 		return;
 	}
 
 	dbus_pending_call_set_notify(call, register_agent_cb, NULL, NULL);
 	dbus_pending_call_unref(call);
+
+	dbus_message_unref(message);
 }
 
 static void unregister_agent(void)
 {
 	DBusMessage *message;
-	const gchar *path = AGENT_PATH;
+	const char *path = AGENT_PATH;
 
 	agent_registered = FALSE;
 
@@ -157,9 +159,9 @@ unregister:
 }
 
 static DBusMessage *create_request_oob_reply(struct btd_adapter *adapter,
-							uint8_t *hash,
-							uint8_t *randomizer,
-							DBusMessage *msg)
+						const uint8_t *hash,
+						const uint8_t *randomizer,
+						DBusMessage *msg)
 {
 	DBusMessage *reply;
 	DBusMessageIter iter;
@@ -196,8 +198,9 @@ static DBusMessage *create_request_oob_reply(struct btd_adapter *adapter,
 	return reply;
 }
 
-static void read_local_complete(struct btd_adapter *adapter, uint8_t *hash,
-					uint8_t *randomizer, void *user_data)
+static void read_local_complete(struct btd_adapter *adapter,
+				const uint8_t *hash, const uint8_t *randomizer,
+				void *user_data)
 {
 	DBusMessage *msg = user_data;
 	DBusMessage *reply;
@@ -296,9 +299,9 @@ static int process_eir(struct btd_adapter *adapter, uint8_t *eir, size_t size,
 
 	ba2str(&eir_data.addr, remote_address);
 
-	DBG("hci%u remote:%s", adapter_get_dev_id(adapter), remote_address);
+	DBG("hci%u remote:%s", btd_adapter_get_index(adapter), remote_address);
 
-	device = adapter_get_device(adapter, remote_address, BDADDR_BREDR);
+	device = adapter_get_device(adapter, &eir_data.addr, BDADDR_BREDR);
 
 	err = check_device(device);
 	if (err < 0) {
@@ -495,9 +498,9 @@ static int process_nokia_com_bt(struct btd_adapter *adapter, void *data,
 		return ret;
 
 	ba2str(&nokia.address, remote_address);
-	DBG("hci%u remote:%s", adapter_get_dev_id(adapter), remote_address);
+	DBG("hci%u remote:%s", btd_adapter_get_index(adapter), remote_address);
 
-	device = adapter_get_device(adapter, remote_address, BDADDR_BREDR);
+	device = adapter_get_device(adapter, &nokia.address, BDADDR_BREDR);
 
 	ret = check_device(device);
 	if (ret != 0) {
@@ -505,7 +508,7 @@ static int process_nokia_com_bt(struct btd_adapter *adapter, void *data,
 		return ret;
 	}
 
-	DBG("hci%u remote:%s", adapter_get_dev_id(adapter), remote_address);
+	DBG("hci%u remote:%s", btd_adapter_get_index(adapter), remote_address);
 
 	if (nokia.name) {
 		adapter_store_cached_name(adapter_get_address(adapter), remote,
@@ -619,7 +622,8 @@ static DBusMessage *push_oob(DBusConnection *conn, DBusMessage *msg, void *data)
 
 	DBG("");
 
-	adapter = manager_get_default_adapter();
+	adapter = btd_adapter_get_default();
+
 	err = check_adapter(adapter);
 	if (err < 0)
 		return error_reply(msg, -err);
@@ -635,7 +639,7 @@ static DBusMessage *push_oob(DBusConnection *conn, DBusMessage *msg, void *data)
 
 	agent = adapter_get_agent(adapter);
 	if (!agent)
-		return error_reply(msg, -ENONET);
+		return error_reply(msg, ENONET);
 
 	io_cap = agent_get_io_capability(agent);
 	agent_unref(agent);
@@ -663,7 +667,8 @@ static DBusMessage *request_oob(DBusConnection *conn, DBusMessage *msg,
 
 	DBG("");
 
-	adapter = manager_get_default_adapter();
+	adapter = btd_adapter_get_default();
+
 	err = check_adapter(adapter);
 	if (err < 0)
 		return error_reply(msg, -err);
@@ -727,7 +732,8 @@ static void neard_appeared(DBusConnection *conn, void *user_data)
 	 * due to neard crash or release before action was completed, postpone
 	 * register until action is finished.
 	 */
-	adapter = manager_get_default_adapter();
+	adapter = btd_adapter_get_default();
+
 	if (adapter && btd_adapter_check_oob_handler(adapter))
 		agent_register_postpone = TRUE;
 	else
