@@ -36,8 +36,8 @@
 #include <sys/stat.h>
 
 #include <bluetooth/bluetooth.h>
-#include <bluetooth/uuid.h>
 
+#include "lib/uuid.h"
 #include "dbus-common.h"
 #include "adapter.h"
 #include "device.h"
@@ -420,17 +420,21 @@ static void property_set_link_loss_level(const GDBusPropertyTable *property,
 	struct monitor *monitor = data;
 	const char *level;
 
-	if (dbus_message_iter_get_arg_type(iter) != DBUS_TYPE_STRING)
-		return g_dbus_pending_property_error(id,
+	if (dbus_message_iter_get_arg_type(iter) != DBUS_TYPE_STRING) {
+		g_dbus_pending_property_error(id,
 					ERROR_INTERFACE ".InvalidArguments",
 					"Invalid arguments in method call");
+		return;
+	}
 
 	dbus_message_iter_get_basic(iter, &level);
 
-	if (!level_is_valid(level))
-		return g_dbus_pending_property_error(id,
+	if (!level_is_valid(level)) {
+		g_dbus_pending_property_error(id,
 					ERROR_INTERFACE ".InvalidArguments",
 					"Invalid arguments in method call");
+		return;
+	}
 
 	if (g_strcmp0(monitor->linklosslevel, level) == 0)
 		goto done;
@@ -478,17 +482,21 @@ static void property_set_immediate_alert_level(
 	struct btd_device *device = monitor->device;
 	const char *level;
 
-	if (dbus_message_iter_get_arg_type(iter) != DBUS_TYPE_STRING)
-		return g_dbus_pending_property_error(id,
+	if (dbus_message_iter_get_arg_type(iter) != DBUS_TYPE_STRING) {
+		g_dbus_pending_property_error(id,
 					ERROR_INTERFACE ".InvalidArguments",
 					"Invalid arguments in method call");
+		return;
+	}
 
 	dbus_message_iter_get_basic(iter, &level);
 
-	if (!level_is_valid(level))
-		return g_dbus_pending_property_error(id,
+	if (!level_is_valid(level)) {
+		g_dbus_pending_property_error(id,
 					ERROR_INTERFACE ".InvalidArguments",
 					"Invalid arguments in method call");
+		return;
+	}
 
 	if (g_strcmp0(monitor->immediatelevel, level) == 0)
 		goto done;
@@ -656,8 +664,6 @@ int monitor_register(struct btd_device *device,
 							attio_disconnected_cb,
 							monitor);
 
-	device_set_auto_connect(device, TRUE);
-
 	return 0;
 }
 
